FixedText

Struct FixedText 

pub struct FixedText<const N: usize> { /* private fields */ }
A Text type which can store at most N bytes from a column.

The data is stored inline the type which typically means on the stack.

Examples

use sqll::{Connection, FixedText, Result};

let c = Connection::open_in_memory()?;

c.execute(r#"
    CREATE TABLE users (name TEXT);

    INSERT INTO users (name) VALUES ('Alice'), ('Bob');
"#)?;

let mut stmt = c.prepare("SELECT name FROM users")?;

let ids = stmt.iter::<FixedText<10>>().collect::<Result<Vec<_>>>()?;
assert_eq!(&ids[0], "Alice");
assert_eq!(&ids[1], "Bob");

Implementations

Source

impl<const N: usize> FixedText<N>

pub const fn new() -> Self

Construct a new empty FixedText.

Examples
use sqll::FixedText;

let s = FixedText::<5>::new();
assert_eq!(s.as_text(), "");
pub const fn from_inner(inner: FixedBlob<N>) -> Self

Converts a vector of bytes to a String without checking that the string contains valid UTF-8.

Examples
use sqll::{FixedBlob, FixedText};

let bytes = FixedBlob::<16>::try_from(&b"Hello World"[..])?;
let s = unsafe { FixedText::from_inner(bytes) };
assert_eq!(s.as_text(), "Hello World");
pub fn as_text(&self) -> &Text

Coerce into the initialized string slice.

Examples
use sqll::{Connection, FixedText};

let c = Connection::open_in_memory()?;

c.execute(r#"
    CREATE TABLE users (name BLOB);

    INSERT INTO users (name) VALUES ('Alice'), ('Bob');
"#)?;

let mut stmt = c.prepare("SELECT name FROM users")?;

assert_eq! {
    stmt.iter::<FixedText<6>>().collect::<Vec<_>>(),
    [Ok(FixedText::<6>::try_from("Alice")?), Ok(FixedText::<6>::try_from("Bob")?)]
};

Methods from Deref<Target = Text>

pub fn as_bytes(&self) -> &[u8]

Get the underlying byte slice for this Text.

Examples
use sqll::Text;

let t = Text::new(b"example");
assert_eq!(t.as_bytes(), b"example");
pub fn to_str(&self) -> Result<&str, Utf8Error>

Attempt to convert this Text into a UTF-8 string slice.

Returns an error if the underlying bytes are not valid UTF-8.

Examples
use sqll::Text;

let t = Text::new(b"example");
assert_eq!(t.to_str()?, "example");

let invalid = Text::new(b"\xF0\x90\x80");
assert!(invalid.to_str().is_err());