pub struct NotThreadSafe { /* private fields */ }Expand description
Error raised when attempting to convert a database object into a thread-safe container, but the database is not configured to be thread-safe.